param([string]$action, [string]$crt, [string]$crt_key, [int]$port = 80, [int]$workers = 1, [string]$mount_path="", [string]$worker_internet_access="n", [string]$admin_password="admin")

function install {
    Param
    (
        [Parameter(Mandatory = $false, Position = 0)]
        [int] $port = 80,
        [Parameter(Mandatory = $false, Position = 1)]
        [string] $crt,
        [Parameter(Mandatory = $false, Position = 2)]
        [string] $crt_key,
        [Parameter(Mandatory = $false, Position = 3)]
        [string] $mount_path,
        [Parameter(Mandatory = $false, Position = 4)]
        [string] $worker_internet_access,
        [Parameter(Mandatory = $false, Position = 5)]
        [string] $admin_password = "admin"
    )

    if ((Get-Content -Path docker-compose.yml) -match "nginx") {
        if ([string]::IsNullOrEmpty($crt)) {
            Write-Warning "We strongly advise against using GAMS Engine without SSL encryption! Only use it in secure, local networks, because everything is sent unencrypted!"
            if ($port -NotMatch "^\d+$") {
                Write-Error "Invalid GAMS Engine port!"
                exit 1
            }
            (Get-Content -Path docker-compose.yml) -replace "\.template-secure", ".template" | Set-Content docker-compose.yml
            $port_replaced = $false
            (Get-Content docker-compose.yml) |
            Foreach-Object {
                if ($_ -match "[0-9]+:[0-9]+") {
                    if ($port_replaced -eq $False) {
                        $port_replaced = $true
                        "      - `"${port}:${port}`""
                    }
                }
                elseif ($_ -match "volumes: #for-ssl") {
                    "    #volumes: #for-ssl"
                }
                elseif ($_ -match "cert.key`" #for-ssl") {
                    '    #  - "$crt-key-file:/etc/nginx/certs/cert.key" #for-ssl'
                }
                elseif ($_ -match "cert.crt`" #for-ssl") {
                    '    #  - "$crt-file:/etc/nginx/certs/cert.crt" #for-ssl'
                }
                else {
                    $_
                }
            } | Set-Content docker-compose.yml

        }
        else {
            if ([string]::IsNullOrEmpty($crt_key)) {
                Write-Error "No key file provided."
                exit 1
            }
            if (-not (Test-Path $crt -PathType leaf)) {
                Write-Error "Certificate file: ${crt} does not exist."
                exit 1
            }
            if (-not (Test-Path $crt_key -PathType leaf)) {
                Write-Error "Certificate key file: ${crt_key} does not exist."
                exit 1
            }
            if ([string]::IsNullOrEmpty($crt_key)) {
                Write-Error "No key file provided."
                exit 1
            }
            if (($port -ne 80) -and ($port -ne 443)) {
                "INFO: Port settings were ignored. Engine will always listen on port 443 when running with SSL configuration!"
            }
            $port = 80
            $crt_forward_slash = $crt -replace "\\", "/"
            $crt_key_forward_slash = $crt_key -replace "\\", "/"
            $port_replaced = $false
            (Get-Content docker-compose.yml) |
            Foreach-Object {
                if ($_ -match "[0-9]+:[0-9]+") {
                    if ($port_replaced -eq $False) {
                        $port_replaced = $true
                        '      - "80:80"'
                        '      - "443:443"'
                    }
                }
                elseif ($_ -match "/etc/nginx/certs/cert.crt") {
                    "      - `"${crt_forward_slash}:/etc/nginx/certs/cert.crt`" #for-ssl"
                }
                elseif ($_ -match "/etc/nginx/certs/cert.key") {
                    "      - `"${crt_key_forward_slash}:/etc/nginx/certs/cert.key`" #for-ssl"
                }
                elseif ($_ -match "#volumes: #for-ssl") {
                    "    volumes: #for-ssl"
                }
                else {
                    $_
                }
            } | Set-Content docker-compose.yml
            (Get-Content -Path docker-compose.yml) -replace "\.template[^-]", '.template-secure"' | Set-Content docker-compose.yml
        }
        if ([string]::IsNullOrEmpty($mount_path)) {
            $nginx_root_url = "/engine"
            $mount_path_trailing = ""
        } else {
            if ($mount_path -match "/") {
                Write-Error "ERROR: Mount location must not include forward slashes"
                exit 1
            }
            $mount_path_trailing = "${mount_path}/"
            $nginx_root_url = ""
        }

        (Get-Content docker-compose.yml) |
        Foreach-Object {
            if ($_ -match "UWSGI_MOUNT_LOCATION: ") {
                "      UWSGI_MOUNT_LOCATION: /${mount_path_trailing}api"
            }
            elseif ($_ -match "mount_url: ") {
                "      mount_url: `"/$mount_path`""
            }
            elseif ($_ -match "mount_api_url: ") {
                "      mount_api_url: `"/${mount_path_trailing}api`""
            }
            elseif ($_ -match "root_url: ") {
                "      root_url: `"/usr/share/nginx$nginx_root_url`""
            }
            elseif ($_ -match "ENGINE_URL: ") {
                "      ENGINE_URL: `"/${mount_path_trailing}api`""
            }
            elseif ($_ -match "BASE_URL: ") {
                "      BASE_URL: `"/${mount_path}`""
            }
            elseif ($_ -match "ENGINE_HTTP_PORT: ") {
                "      ENGINE_HTTP_PORT: ${port}"
            }
            else {
                $_
            }
        } | Set-Content docker-compose.yml
    }
    $worker_service_found = $false
    $start_replace = $false
    (Get-Content docker-compose.yml) |
    Foreach-Object {
        if ($start_replace -eq $true){
            $start_replace= $false
            $worker_service_found = $false
            if($_ -match "worker-ext-network" -and $worker_internet_access -eq "n"){
            }elseif($_ -match "worker-network" -and $worker_internet_access -eq "y"){
                "      - worker-ext-network"
                $_
            }else{
                $_
            }
        }
        elseif($_ -match " worker:"){
            $worker_service_found = $true
            $_
        }
        elseif($_ -match " networks:" -and $worker_service_found -eq $true){
            $_
            $start_replace = $true
        }
        else {
            $_
        }
    } | Set-Content docker-compose.yml
    docker pull gams/engine-pre-migrate:latest
    if (!$?) { exit 1 }
    docker run -e "admin_password=${admin_password}" -v "$(Get-Location):/home/mig_user/release" --rm gams/engine-pre-migrate:latest
    if (!$?) { exit 1 }
    pull_images
    "GAMS Engine installed successfully!"
}

function pull_images {
    "Pulling Docker images for GAMS Engine"
    docker compose -f docker-compose.yml pull
    if (!$?) { exit 1 }
}

function prune {
    "Doing some housekeeping..."
    docker image prune -f
    if (!$?) { exit 1 }
}

function launch {
    Param
    (
        [Parameter(Mandatory = $false, Position = 0)]
        [int] $workers = 1
    )

    if ($workers -NotMatch "^\d+$") {
        "Invalid number of workers!"
        exit 1
    }
    "Starting GAMS Engine"
    docker compose -f docker-compose.yml up -d --scale worker=$workers
    if (!$?) { exit 1 }
    "GAMS Engine started."
}

function stop {
    "Stopping GAMS Engine"
    docker compose -f docker-compose.yml down
    if (!$?) { exit 1 }
    "GAMS Engine stopped."
}

function restart {
    "Restarting GAMS Engine"
    docker compose -f docker-compose.yml restart
    if (!$?) { exit 1 }
    "GAMS Engine restarted."
}

function uninstall {
    "Uninstalling GAMS Engine"
    $uninstall_confirm = Read-Host -Prompt "Are you sure you want to remove all data of GAMS Engine? This cannot be undone! (yes)"

    if ($uninstall_confirm -ne "yes") {
        echo "Uninstalling GAMS Engine was interrupted."
        exit 0
    }
    stop
    docker compose -f docker-compose.yml down -v
    if (!$?) { exit 1 }
    "GAMS Engine uninstalled."
}

switch -Exact ($action) {
    "stop" {
        stop
        break
    }
    "uninstall" {
        uninstall
        break
    }
    "restart" {
        restart
        break
    }
    "start" {
        launch $workers
        break
    }
    "install" {
        install $port $crt $crt_key $mount_path $worker_internet_access $admin_password
        break
    }
    "update" {
        pull_images
        launch
        prune
        break
    }
    Default {
        "Usage: {0} install -port -crt -crt_key -mount_path -worker-internet-access|start -workers|stop|restart|update|uninstall" -f $MyInvocation.MyCommand.Name
        exit 1
    }
}

exit 0
